#!/usr/bin/ruby
require "json"
require "thread"
BRAIN = "./algo"

def parse_spec(txt_spec)
  first, *others = txt_spec.split(/\n/)
  if first =~ /([0-9]+)\s+([0-9]+)\s+([PD])(B?)/
    width = $1.to_i
    height = $2.to_i
    full = $3 == "P"
    colored = $4 == "B"
  else
    raise "The input seems to be incorrect :("
  end

  json = {
    width: width,
    height: height,
    full: full,
    colored: colored,
    bricks: []
  }

  others.each do |line|
    if line =~ /([ARGBY])\s+([0-9]+)[xX]([0-9]+)\s+([0-9]+)/
      color = case $1
              when 'A' then 0
              when 'R' then 1
              when 'G' then 2
              when 'B' then 3
              when 'Y' then 4
              else
                raise "Bad brick color"
              end
      dimA = $2.to_i
      dimB = $3.to_i
      count = $4.to_i
      brick_ty = case [dimA, dimB]
                 when [2, 2] then 0
                 when [1, 1] then 1
                 when [1, 2], [2, 1] then 2
                 when [1, 3], [3, 1] then 3
                 when [1, 4], [4, 1] then 4
                 else 
                   raise "Bad brick dimensions"
                 end
      json[:bricks] << {color: color, brick: brick_ty, count:count}
    else
      raise "The input is incorrect :/"
    end
  end

  json.to_json
end


input_dir = ARGV[0] || "."
files = []
Dir.glob("#{input_dir}/*.in").sort.reverse.each do |file|
  files << file
end
files_mutex = Mutex.new

threads = (0...4).map do
  Thread.new do
    loop do
      file = nil
      files_mutex.synchronize do
        file = files.pop
      end

      if file.nil?
        break
      end

      spec_json = parse_spec(File.read file)
      io = IO.popen(BRAIN, "r+")
      io.write(spec_json)
      io.close_write
      result = io.read
      io.close

      unless result.empty?
        File.write(file.gsub(/\.in$/, ".49"), result)
      end
      puts file
    end
  end
end

threads.each(&:join)
